from graphene_django.types import DjangoObjectType
from django.contrib.auth import get_user_model


User = get_user_model()


class UserType(DjangoObjectType):
    class Meta:
        model = User
        exclude = ['password']



from celery import shared_task
from .utils import (
    send_register_email, 
    send_reset_password_email,
    send_change_email
)


@shared_task(ignore_result=True)
def task_send_register_email(id, email, username, first_name, last_name):
    send_register_email(id, email, username, first_name, last_name)


@shared_task(ignore_result=True)
def task_send_reset_password_email(id, email, username, first_name, last_name):
    send_reset_password_email(id, email, username, first_name, last_name)


@shared_task(ignore_result=True)
def task_send_change_email(id, email, username, first_name, last_name):
    send_change_email(id, email, username, first_name, last_name)



from django.http import response
import graphene
import graphql_jwt
from graphene.types.generic import GenericScalar
from graphql_jwt.decorators import login_required

from django.contrib.auth import get_user_model
from django.core.validators import validate_email
from django.core.exceptions import ValidationError
    
from .types import UserType
from .utils import get_from_redis, token_delete_to_redis
from .tasks import task_send_register_email, task_send_change_email, task_send_reset_password_email


User = get_user_model()


class AccountInput(graphene.InputObjectType):
    username = graphene.String(required=False)
    email = graphene.String(required=True)
    password = graphene.String(required=True)
    first_name = graphene.String(required=False)
    last_name = graphene.String(required=False)


class ChangeEmail(graphene.Mutation):
    class Arguments:
        email = graphene.String()
    
    response = GenericScalar()

    @login_required
    def mutate(parent, info, email):
        try:
            User.objects.get(email=email)
            return ChangeEmail(response={'status': 'error', 'message': 'Email address already taken.'})

        except User.DoesNotExist:
            user = info.context.user
            user.email = email
            user.is_active = False
            user.save()
            task_send_change_email.delay(      
                id=user.id , 
                email=user.email, 
                username=user.username, 
                first_name=user.first_name, 
                last_name=user.last_name
                )
            return ChangeEmail(response={'status': 'success', 'message': 'send email .'})


class VerifyEmail(graphene.Mutation):
    class Arguments:
        token = graphene.String(required=True)
        email = graphene.String(required=True)
    
    response = GenericScalar()

    def mutate(parent, info, token, email):
        try:
            user = User.objects.get(email=email)
            token_from_redis = get_from_redis(user.id, 'change_email')

            if not token_from_redis:
                return VerifyEmail(response={'status': 'error', 'message': 'Wrong/Expired Token!'})

            if token != token_from_redis.decode('UTF-8'):
                return VerifyEmail(response={'status': 'error', 'message': 'Wrong/Expired Token!'})

            user.is_active = True
            user.save()
            token_delete_to_redis(user.id, 'change_email')
            return VerifyEmail(response={'status': 'success', 'message': 'verify email.'})

        except User.DoesNotExist:
            return VerifyEmail(response={'status': 'error', 'message': 'Wrong/Expired Token!'})


class ResetPassword(graphene.Mutation):
    class Arguments:
        email = graphene.String()
    
    response = GenericScalar()

    def mutate(parent, info, email):
        try:
            user = User.objects.get(email=email)
            task_send_reset_password_email.delay(
                id=user.id , 
                email=user.email, 
                username=user.username, 
                first_name=user.first_name, 
                last_name=user.last_name
            )
            return ResetPassword(response={'status': 'success', 'message': 'send email.'})

        except User.DoesNotExist:
            return ResetPassword(response={'status': 'error', 'message': 'Email dose not exists.'})
        

class VerifyPassword(graphene.Mutation):
    class Arguments:
        token = graphene.String(required=True)
        email = graphene.String(required=True)
        new_password = graphene.String(required=True)
    
    response = GenericScalar()

    def mutate(parent, info, token, email, new_password):
        try:
            user = User.objects.get(email=email)
            token_from_redis = get_from_redis(user.id, 'reset_password')

            if not token_from_redis:
                return VerifyPassword(response={'status': 'error', 'message': 'Wrong/Expired Token!.'})

            if token != token_from_redis.decode('UTF-8'):
                return VerifyPassword(response={'status': 'error', 'message': 'Wrong/Expired Token!.'})

            user.set_password(new_password)
            user.save()
            token_delete_to_redis(user.id, 'reset_password')
            return VerifyPassword(response={'status': 'success', 'message': 'verify password.'})

        except User.DoesNotExist:
            return VerifyPassword(response={'status': 'error', 'message': 'Wrong/Expired Token!.'})


class ChangePassword(graphene.Mutation):
    class Arguments:
        old_password = graphene.String()
        new_password = graphene.String()
    
    response = GenericScalar()

    @login_required
    def mutate(parent, info, old_password, new_password):	
        user = info.context.user
        if user.check_password(old_password):
            user.set_password(new_password)
            user.save()
            return ChangePassword(response={'status': 'success', 'message': 'change password.'})
        else:
            return ChangePassword(response={'status': 'error', 'message': 'Your old password is invalid.'})
            

class CreateAccount(graphene.Mutation):
    class Arguments:
        input = AccountInput(required=True)
    
    user = graphene.Field(UserType)
    response = GenericScalar()

    def mutate(parent, info, input):
        user=None
        try:
            validate_email(input.email)
            try: 
                user_exists = User.objects.get(email=input.email)
                if user_exists.is_active:
                    return CreateAccount(response={'status': 'error', 'message': 'Duplicate user.'})
                else:
                    user = user_exists

            except User.DoesNotExist:
                user = User.objects.create_user(
                    username=input.username, 
                    email=input.email, 
                    password=input.password, 
                    is_active=False,
                    first_name=input.first_name,
                    last_name=input.last_name
                    )

            task_send_register_email.delay(
                id=user.id , 
                email=user.email, 
                username=user.username, 
                first_name=user.first_name, 
                last_name=user.last_name)
            return CreateAccount(user=user, response={'status': 'success', 'message': 'create user.'})

        except ValidationError:
            return CreateAccount(user=user, response={'status': 'error', 'message': 'Enter a valid e-mail address.'})
       
        
class ActivateAccount(graphene.Mutation):
    class Arguments:
        token = graphene.String(required=True)
        email = graphene.String(required=True)

    response = GenericScalar()

    def mutate(parent, info, token, email):
        try:
            user = User.objects.get(email=email)
        except User.DoesNotExist:
            return ActivateAccount(response={'status': 'error', 'message': 'Wrong/Expired Token!.'})

        token_from_redis = get_from_redis(user.id, 'register')
        if not token_from_redis:
            return ActivateAccount(response={'status': 'error', 'message': 'Wrong/Expired Token!.'})

        if token != token_from_redis.decode('UTF-8'):
            return ActivateAccount(response={'status': 'error', 'message': 'Wrong/Expired Token!.'})

        user.is_active = True
        user.save()
        token_delete_to_redis(user.id, 'register')
        return ActivateAccount(response={'status': 'success', 'message': 'active account.'})


class UpdateAccount(graphene.Mutation):
    class Arguments:
        first_name = graphene.String()
        last_name = graphene.String()
        username = graphene.String()

    response = GenericScalar()
    user = graphene.Field(UserType)

    @login_required
    def mutate(parent, info, first_name, last_name, username):
        user = User.objects.get(id=info.context.user.id)
        user.first_name = first_name or user.first_name
        user.last_name = last_name or user.last_name
        user.username = username or user.username
        user.save()
        return UpdateAccount(user=user, response={'status': 'success', 'message': 'update account.'})


class AccountsMutation(graphene.ObjectType):
    token_auth = graphql_jwt.ObtainJSONWebToken.Field()
    verify_token = graphql_jwt.Verify.Field()
    refresh_token = graphql_jwt.Refresh.Field()

    create_account = CreateAccount.Field()
    activate_account = ActivateAccount.Field()
    update_account = UpdateAccount.Field()

    change_password = ChangePassword.Field()
    reset_password = ResetPassword.Field()
    verify_password = VerifyPassword.Field()

    change_email = ChangeEmail.Field()
    verify_email = VerifyEmail.Field()




from django.contrib.auth.models import BaseUserManager, AbstractBaseUser
from django.contrib.auth.models import PermissionsMixin
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone
from django.db import models


class EmailUserManager(BaseUserManager):
    def _create_user(self, email, password, is_staff, is_superuser, **extra_fields):
        """
        Creates and saves a User with a given email and password.
        """
        now = timezone.now()
        if not email:
            raise ValueError('Users must have an email address.')
        email = self.normalize_email(email)
        user = self.model(
            email=email,
            is_staff=is_staff, 
            is_active=True,      
            is_superuser=is_superuser,
            last_login=now, 
            date_joined=now, 
            **extra_fields
            )
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, email, password=None, **extra_fields):
        return self._create_user(
            email, 
            password, 
            False, 
            False, 
            False,
            **extra_fields
            )

    def create_superuser(self, email, password, **extra_fields):
        return self._create_user(
            email, 
            password, 
            True, 
            True, 
            True,
            **extra_fields
            )


class EmailAbstractUser(AbstractBaseUser, PermissionsMixin):
    """
    An abstract base class implementing a fully featured User model with
    admin-compliant permissions.
    Email and password are required. Other fields are optional.
    """
    first_name = models.CharField(_('first name'), max_length=30, blank=True)
    last_name = models.CharField(_('last name'), max_length=30, blank=True)
    email = models.EmailField(_('email address'), max_length=255, unique=True)
    is_staff = models.BooleanField(
        _('staff status'), default=False,
        help_text=_('Designates whether the user can log into this '
                    'admin site.'))
    is_active = models.BooleanField(
        _('active'), default=True,
        help_text=_('Designates whether this user should be treated as '
                    'active.  Unselect this instead of deleting accounts.'))
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)


    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = ['first_name', 'last_name']

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')
        abstract = True

    def get_full_name(self):
        """
        Returns the first_name plus the last_name, with a space in between.
        """
        full_name = '%s %s' % (self.first_name, self.last_name)
        return full_name.strip()

    def get_short_name(self):
        "Returns the short name for the user."
        return self.first_name

    def __str__(self):
        return self.email



import os
import binascii

from django.conf import settings
from django.template.loader import render_to_string
from django.core.mail.message import EmailMultiAlternatives


redis = settings.REDIS


def _generate_code():
	return binascii.hexlify(os.urandom(20)).decode('utf-8')


def send_multi_format_email(template_prefix, template_context, target_email):
	subject_file = 'accounts/%s_subject.txt' % template_prefix
	txt_file = 'accounts/%s.txt' % template_prefix
	html_file = 'accounts/%s.html' % template_prefix

	subject = render_to_string(subject_file).strip()
	from_email = settings.EMAIL_HOST_USER
	to = target_email
	text_content = render_to_string(txt_file, template_context)
	html_content = render_to_string(html_file, template_context)
	msg = EmailMultiAlternatives(subject, text_content, from_email, [to])
	msg.attach_alternative(html_content, 'text/html')
	msg.send()


def token_add_to_redis(id, mode):
    token = _generate_code()
    name = f'{id}_{mode.lower()}'
    redis.set(name=name, value=token, ex=14400)
    return token


def token_delete_to_redis(id, mode):
    name = f'{id}_{mode.lower()}'
    redis.delete(name)


def get_from_redis(id, mode):
    name = f'{id}_{mode.lower()}'
    return redis.get(name=name)


def send_register_email(id,  email, username, first_name, last_name):
    token_delete_to_redis(id, 'register')
    ctxt = {
        'email': email,
        'first_name': first_name,
        'last_name': last_name,
        'token': token_add_to_redis(id, 'register'),
        'btn_name': 'Verify your email . Click Me !'
    }
    send_multi_format_email('signup_email', ctxt, target_email=email)


def send_reset_password_email(id, email, username, first_name, last_name):
    token_delete_to_redis(id, 'reset_password')
    ctxt = {
        'email': email,
        'username': username,
        'first_name': first_name,
        'last_name': last_name,
        'token': token_add_to_redis(id, 'reset_password'),
        'btn_name': 'Reset password . Click Me !'
    }
    send_multi_format_email('signup_email', ctxt, target_email=email)


def send_change_email(id, email, username, first_name, last_name):
    token_delete_to_redis(id, 'change_email')
    ctxt = {
        'email': email,
        'username': username,
        'first_name': first_name,
        'last_name': last_name,
        'token': token_add_to_redis(id, 'change_email'),
        'btn_name': 'Change email . Click Me !'
    }
    send_multi_format_email('signup_email', ctxt, target_email=email)



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='django-graphql-accounts',
    version='0.12',
    author='Mahdi Asadzadeh',
    author_email='mahdi.asadzadeh.programing@gmail.com',
    description='A GraphQl API for user signup and authentication using email addresses',
    keywords=[
        'django', 'python', 'graphql', 'GraphQL', 'django-graphone', 'Graphone', 'api',
        'auth', 'authentication', 'email', 'user', 'username', 'python-graphone',
        'registration', 'signup', 'login', 'logout', 'password', 'django-registration',
        'django-email-as-username'
    ],
    url='https://github.com/mahdi-asadzadeh/django-graphql-accounts',
    download_url='https://github.com/mahdi-asadzadeh/django-graphql-accounts',
    packages=['django_graphql_accounts'],
    include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[
        'Django>=2.2.8,<=3.1,<=3.2',
        'graphene-django>=2.15' ,
        'celery>=5.1',
        'django-graphql-jwt>=0.3',
        'redis>=3.5'
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Framework :: Django :: 3.2',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development',
        'Topic :: Utilities',
    ],
)



from graphql_jwt.decorators import login_required
from django.contrib.auth import get_user_model
from .types import UserType
import graphene


User = get_user_model()


class AccountsQuery(graphene.ObjectType):
    account = graphene.Field(UserType)

    @login_required
    def resolve_account(parent, info):
        user = info.context.user
        return User.objects.get(id=user.id)
        